/*
 * Thanks to Lars H. Andersson, who submitted the first idea of this simple function
 */
 
#include <pro/sntp.h>
#include <sys/socket.h>
#include <sys/heap.h>
#include <string.h>
#include "../crt/ctime.h"
#include <stdio.h>
#include <sys/thread.h>
#include <sys/timer.h>

typedef struct _sntpframe sntpframe;
struct _sntpframe {
	u_char mode;
	u_char stratum;
	u_char poll;
	u_char precision;
	u_long root_delay;
	u_long root_dispersion;
	u_long reference_identifier;
	u_long reference_ts_sec;
	u_long reference_ts_frac;
	u_long originate_ts_sec;
	u_long originate_ts_frac;
	u_long receive_ts_sec;
	u_long receive_ts_frac;
	u_long transmit_ts_sec;
	u_long transmit_ts_frac;
};

#define NTP_PORT	123
#define SNTP_PORT NTP_PORT

struct SNTP_resync_args
{
	u_long server_addr;
	u_long interval;
};

static THREAD(SNTP_resync, arg)
{
	u_long server_addr = ((struct SNTP_resync_args*)arg)->server_addr;
	u_long interval = ((struct SNTP_resync_args*)arg)->interval;
	u_long cur_server_addr = server_addr;
	NutHeapFree (arg);
	
	NutThreadSetPriority (63);
	int retry = 0;
	time_t t;
	for (;;)
	{
		if (NutSNTPGetTime (&cur_server_addr, &t))
		{	// if any error retry
			if (cur_server_addr != server_addr && server_addr == 0xFFFFFFFF)
			{
				cur_server_addr = server_addr;
				continue;
			}
			
			if (retry++ >= 3)	// if numer of retries >= 3 wait 30 secs before next retry sequence ...
			{
				retry = 0;
				NutSleep (30000);
			}
			else			// ... else wait 5 secs for next retry
				NutSleep (5000);
		}
		else
		{	// no error
			stime (&t);			// so set the time
			retry = 0;
			NutSleep (interval);		// and wait the interval time
		}
	}
}

int NutSNTPGetTime (u_long *server_adr, time_t *t)
{
	//first check the pointers
	u_long rec_addr;
	if (t == NULL)
		return -1;
	if (server_adr == NULL)
		return -1;
		
	UDPSOCKET *sock = NULL;	// the udp socket
	sntpframe *data;		// we're using the heap to save stack space
	u_short port;		// source port from incoming packet
	int len;
	int result = -1;
	
	if ((data = NutHeapAlloc (sizeof (*data))) == NULL)
		goto error;
		
	sock = NutUdpCreateSocket (0);	// allocate new udp socket
	if (sock == NULL)
		goto error;
	memset (data, 0, sizeof (*data));    // initialize data
	
	data->mode = 0x1B;			// LI VN and Mode bit fields (all in u_char mode);
	if (NutUdpSendTo (sock, *server_adr, SNTP_PORT, data, sizeof (*data)))	// Send packet to server
		goto error;				// on error return -1
	
	len = NutUdpReceiveFrom (sock, &rec_addr, &port, data, sizeof(*data), 5000);	// Receive packet with timeout of 5s
	if (len <= 0)
		goto error;			// error or timeout occured
	
	if (port != SNTP_PORT || (data->mode & 0xc0) == 0xc0)		// if source port is not SNTP_PORT or server is not in sync return
		goto error;

	*t = ntohl (data->transmit_ts_sec) - (70 * 365 + _LEAP_YEAR_ADJUST) * _DAY_SEC;
	*server_adr = rec_addr;
	result = 0;
error:
	if (sock) NutUdpDestroySocket (sock);
	if (data) NutHeapFree (data);
	return result;
			
}

int NutSNTPStartThread(u_long server_addr, u_long interval)
{
	struct SNTP_resync_args *arg = NutHeapAlloc (sizeof (struct SNTP_resync_args));
	arg->server_addr = server_addr;
	arg->interval = interval;
	if (NutThreadCreate ("sntp", SNTP_resync, arg, 384))
		return 0;
	else
		return -1;
}
